<?php
//    Copyright 2013 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	require_once("acl.php");
	require("config.inc");
	require("get_vo_configuration.php");
	require("https_client_auth.php");

	// define base API handler class
	abstract class VOMSAPI {
        protected $dnformat = "legacy";

		abstract protected function request_failed ($e_name, $e_descr);

        // error names
        abstract protected function error_authException ();
        abstract protected function error_userException ();
        abstract protected function error_methodException ();
        
        // customized error description prefixes
        protected function acl_prefix      () { return ""; }
        protected function syntax_prefix   () { return ""; }
        protected function nogroup_prefix  () { return ""; }
        protected function norole_prefix   () { return ""; }
        protected function nouser_prefix   () { return ""; }
        protected function noattribute_prefix () { return ""; }
        protected function nomethod_prefix () { return ""; }

        // runtime customization
        public function set_DNformat ( $format ) { $this->dnformat = $format; }
        public function formatDN ( $dn ) {
            if ( $this->dnformat == "rfc" )     return DNtoRFC2253($dn);
            if ( $this->dnformat == "rfc2253" ) return DNtoRFC2253($dn);
            return $dn;
        }

		// define unified permissions checking interface
		protected function acl_check ($perm, $op, $rw) {
			if ( ! isACLallow($perm, $op, $rw) ) {
				$this->request_failed($this->error_authException(),
					sprintf("%soperation is not allowed due to %s:%s ACL", $this->acl_prefix(), $op, $rw));
			}
		}
		
		// check name for not allowed characters
		protected function name_check ($argname) {
			if ( ! preg_match('/^[a-zA-Z0-9\-\_\.]+$/', $argname) ) {
				$this->request_failed($this->error_userException(),
					sprintf("%sSyntax error in name passed: %s", $this->syntax_prefix(), $argname));
			}
		}

		// check group name contains /vo part and return array of group hierarchy
		protected function parse_group_name ($group_name, $nosyntax = false ) {
			$gname_arr = explode('/',$group_name);
			if ( ! $nosyntax ) if ( $gname_arr[1] !== $this->vo ) {
				$this->request_failed($this->error_userException(),
					sprintf("%sSyntax error in group name: %s", $this->syntax_prefix(), $group_name));
			}
			return $gname_arr;
		}

		protected function parse_container ($container) {
			$cont_arr = explode('/',$container);
			$capability = NULL;
			$role = NULL;
			$groups = array ();
			// get capability if any
			$contp = array_pop($cont_arr);
			if ( $contp ) {
				if ( preg_match('/^C(apability|APABILITY)=/',$contp) ) {
					$capability = preg_replace('/^C(apability|APABILITY)=/','',$contp);
					if ( $capability == 'NULL' ) $capability = NULL;
					$contp = array_pop($cont_arr); 
				}
			} else return NULL;
			// get role if any
			if ( $contp ) { 
				if ( preg_match('/^R(ole|OLE)=/',$contp) ) {
					$role = preg_replace('/^R(ole|OLE)=/','',$contp);
					if ( $role == 'NULL' ) $role = NULL;
					$contp = array_pop($cont_arr);
				} 
			} else return NULL;
			// get all groups array
			while ( $contp ) {
				$ccontp = preg_replace(array('/^VO=/','/^G(roup|ROUP)=/'),array('',''),$contp);
				array_unshift($groups,$ccontp);
				$contp = array_pop($cont_arr);
			}
			return  array (	"groups" => $groups,
					"role" => $role,
					"capability" => $capability );
		}

		// get group ID by name
		protected function get_gid ($group_name) {
			$gid = getGroupByName($group_name);
			if ( ! $gid ) $this->request_failed($this->error_userException(),
				sprintf("%sGroup '%s' is not defined for this vo.", $this->nogroup_prefix(), $group_name));	
			return $gid;
		}

		// get role ID by name
		protected function get_rid ($role_name) {
			$rname = preg_replace(array('/Role=/','/ROLE=/'), array('',''), $role_name);
			$rid = getRoleId($rname);
			if ( ! $rid ) $this->request_failed($this->error_userException(),
				sprintf("%sRole '%s' is not defined for this vo.", $this->norole_prefix(), $rname));
			return $rid;
		}

		// get user ID by DN/CA
		protected function get_uid ($dn, $ca) {
			$uid = getUserID($dn, $ca);
			if ( ! $uid ) $this->request_failed ($this->error_userException(),
				sprintf("%sUser %s,%s is not found in VO database", $this->nouser_prefix(), $dn, $ca));
			return $uid;
		}

		// get attribute ID by name
		protected function get_aid ($attr_name) {
			$aid = getAttributeId ($attr_name);
			if ( ! $aid ) $this->request_failed ($this->error_userException(),
				sprintf("%sAttribute %s is not found in VO database", $this->noattribute_prefix(), $attr_name));
			return $aid;
		}

		// unsuported methods exception
		public function __call($method, $args) {
			$this->request_failed ($this->error_methodException(), 
				sprintf("%sRequested method %s(%s) is not supported", $this->nomethod_prefix(), 
					$method, implode(', ', $args)));
		}

	}

?>
